#define F_CPU 16000000

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>
#include "usart.h"

#define INSIDE 1
#define OUTSIDE 0
#define PERIMETER 30

#define MIN_POW 180
#define MAX_POW 220

#define CHANGE_DIR MIN_POW
#define stop_motor 0
#define VAL 650


volatile float distanceCM;
volatile int check = 0;
volatile int location = 1;

 int sum;
 int found;


//Senzorul de distanta se afla pe PORTC
#define TRIGGER PC4
#define ECHO PC3

/* Senzor distanta initializari */
void distance_sensor_init(void)
{
	DDRC &= ~(1 <<  ECHO);  //echo -> input pin(0) -> citesc de aici
	DDRC |= (1 << TRIGGER); //trigger -> output pin(1)

	//Intrerupere pentru a prelua datele de la senzor
	PCMSK2 = (1 << PCINT19);
	PCICR = (1 << PCIE2); // intre PCINT23:16
	sei();
}

/*Senzor distanta - control trigger */
void trigger(void)
{
    //trigger trebuie tinut pe high 10us
    PORTC |= (1 << TRIGGER);
    _delay_us(15);
    //apoi pus pe low din nou
    PORTC &= ~(1 << TRIGGER);
}

/* Intrerupere pentru citire date de la echo -> TIMER 1
** check se schimba cand se schimba starea lui echo (low<->high) si se produce o intrerupere
*/
ISR(PCINT2_vect)
{

	if( !check) {
		//Normal mode, cu prescaler de 8 si resetarea registrului contor
		TCCR1A = 0X00;
		TCCR1B = (1 << CS11);
		TCNT1 = 0x00;
		check = 1;

	} else if (check){

		TCCR1B = 0x00;
		check = 0;
		/*TCNT1 e registrul contor, 340m/s e viteza sunetului => 0.034cm/us ca sa pastrez unitatile de masura
		** Distanta = Timp * viteza /2, unde timpul este contorul timerului, viteza e viteza sunetului in aer, iar drumul e dus-intors
		*/
		distanceCM = (TCNT1 * 0.034) / 4;
	}
}

//SENZOR LINIE

/* Senzorul de linie  foloseste ADC -> functii lab 5*/
void ADC_init(void)
{
	ADMUX = (1 << REFS0);
	// prescaler la 128
	ADCSRA = (1 << ADEN) | (7 << ADPS0);
}

int ADC_get(uint8_t channel)
{
	// conversie ADC pe canalul "channel"
	ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;
	ADCSRA |= (1 << ADSC);
	// asteapta completarea
	while(ADCSRA & (1 << ADSC));
	return ADC;
}


//MOTOARE
void initialize_motor(void)
{
	//Conectat la driverul de motoare
	PORTB = 0;
	DDRA = 0;
	PORTA = 0xFF;
	//Setare pini intrate/iesire
	DDRB |= (1 << PB0) | (0 << PB1);
	DDRB |= (1 << PB6) | (0 << PB7);
	//Enable A si EnableB -> Enable PWM pt motoarele A si B
	DDRB |= (1 << PB3) | (1 << PB4);
	//Sens de miscare inainte
	PORTB |= (1 << PB7);
	PORTB |= (1 << PB0);
}

void start_pwm(void)
{
	//fast pwm cu top la oxff
	TCCR0A |= (1 << COM0A1) | (1 << COM0B1) | (1 << WGM01) | (1 << WGM00);
	// prescaler 1
	TCCR0B |= (1 << CS00);
	// fara intreruperi de match sau overflow
	TIMSK0 = 0;

}


void go_forward(void)
{
	OCR0A = 200;
	OCR0B = 200;
}


//Valori intre 0-255 pt ca sunt pe timer 0
void go_left(void)
{
	OCR0A = 170;
	OCR0B = 215;
}

void go_right(void)
{
	OCR0B = 0;
	OCR0A = 215;
}

void go_hard_left(void) {
	OCR0A = 0;
	OCR0B = 215;
}

//Opreste motoarele
void stop(void)
{
	OCR0A = 0;
	OCR0B = 0;
}
void stay_inside()
{
	int i;
	
	sum = 0;
	

	for(i = 0; i < 3; i++) {
		if (ADC_get(i) > VAL) {
			sum--;
			found = 0;
		}
	}

	if(sum < 0) {
		go_hard_left();
	} else {
		go_forward();
	}
	
}

void find_object(void)
{
	found = 0;
	while(found == 0) {
		trigger();
		//_delay_ms(10);
		stay_inside();
		if(distanceCM > PERIMETER){
			go_left();
			//_delay_ms(1000);
			go_forward();
			_delay_ms(1000);
			go_right();
			//_delay_ms(1000);
			go_forward();
			//_delay_ms(1000);
		}else {
			found = 1;
			stop();
			break;
		}
	}

}




//Inlatura obiectul
void destroy(void)
{
	if(found == 1) {
		go_forward();
		stay_inside();
	} else {
		find_object();
	}
}



int main() {

	initialize_motor();
	start_pwm();
	ADC_init();      //linie
	
	distance_sensor_init(); // initializari senzor distanta

	
	while(1) {
		
		stay_inside();
		trigger();
		if(distanceCM < 30) {
			go_forward();
		}
	}	
	return 0;

}
